/*
 * IDLflex -- A flexible, generic code generator for CORBA IDL
 *
 * Copyright (C) 2000  Hans Reiser, AspectIX Research Team
 *                     University of Erlangen-Nuernberg, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 * The full text of the license can be found in the LICENSE.txt file.
 *
 * $Id: IDLFileWriter.java,v 1.2 2000/12/15 15:32:25 fzhauck Exp $
 */

package org.aspectix.IDLflex;

import java.io.File;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.Writer;
import java.io.IOException;

/** base class for creating and writing into a file.
 */
public class IDLFileWriter implements IDLWriter {
    Writer out;
    String buffer;
    static java.util.Vector reopenFiles;
    static {
	reopenFiles = new java.util.Vector();
    }

    /** public constructor, which automatically creats the file of the
     *  given name.
     */
    public IDLFileWriter(String name, String mode) throws IOException {
	this(new File(name), mode);
    }
    /** public constructor, which automatically creats the file
     *  represented by a File object.
     */
    public IDLFileWriter(File file, String mode) throws IOException {
	this();
	open(file, mode);
    }
    /** public constructor which creats an empty IDLFileWriter; to be
     *  used with an subsequent initialization by the 'open' methode.
     */
    public IDLFileWriter() {
	out=null;
	buffer = "";
    }
    /** initialization: opens the file specified by its name.
     *  (creats a File object and then calls the other open method)*/
    public void open(String name, String mode) throws IOException {
	open(new File(name), mode);
    }
    /** initialization: opens the file specified by a File object.
     *  
     *  if necessary, creates all required directories.
     *  @exception IOException if the necessary directories cannot be
     *        created, or if opening the file for writing fails.
     */
    public void open(File file, String mode) throws IOException {
	boolean append=false;
	String pathstr= file.getParent();
	if(pathstr!=null) {
	    File path = new File(file.getParent());
	    if(!path.exists()) {
		if( path.mkdirs()==false ) {
		    throw new IOException("Cant create directories");
		}
	    }
	}
	if(mode.equals("")||mode.equals("CREATE")) append=false;
	else if(mode.equals("APPEND")) append=true;
	else if(mode.equals("REOPEN")) append=reopenFiles.contains(file);
	reopenFiles.addElement(file);
	out = new BufferedWriter(new FileWriter( file.getPath(), append ));
    }
    /** writes a String into the file... */
    public void write(String s) throws IOException {
	//if(s.startsWith("\n")) s=s.substring(1);
	//if(s.endsWith("\n")) s=s.substring(0,s.length()-1);
	//// s=" >"+s.trim()+"< ";
	buffer+=s;
	int pos;
	while( (pos = buffer.indexOf("\n")) >=0 ) {
	    out.write(buffer.substring(0,pos+1));
	    buffer=buffer.substring(pos+1);
	}
    }
    /** flushes the internal buffer and closes the file.
     *  It is essential to call the close methode, otherwise the
     *  internal buffer will not be written to the file.
     */
    public void close() throws IOException {
	out.write(buffer);
	out.close();
    }
}
