/*
 * IDLflex -- A flexible, generic code generator for CORBA IDL
 *
 * Copyright (C) 2000  Hans Reiser, AspectIX Research Team
 *                     University of Erlangen-Nuernberg, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 * The full text of the license can be found in the LICENSE.txt file.
 *
 * $Id: IDLflexException.java,v 1.4 2001/09/07 09:20:42 reiser Exp $
 */

package org.aspectix.IDLflex;

import org.omg.CORBA.*;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.aspectix.IDLflex.IRObj.*;

/** Exception thrown for all internal errors of IDLflex. 
 *  The code prints the error message, the current location in the
 *  IDL object representation, and the path through the XML document
 *  which leads to the element which was active when the error occured.
 */
public class IDLflexException extends java.lang.Exception {
    IDLObject obj;
    Node xmlnode;
    public IDLflexException(String s) {
	super( constructMessage(s) );
    }
    static final String constructMessage(String s) {
	String a="\n"+s+"\n";
	IDLDefCompiler.IDLObjectVector v;

	if(IDLflexMain.c==null) return a;
	v=IDLflexMain.c.generatePath;
	if(v==null) return a;

	
	// Path to Interface Repository object
	int c = v.count_objects();
	if(c>0) {
	    a=a+"\nCurrent interface repository object: \n";
	    a+=printObjectPath(v);
	}

	// Path in the XML document
	c=v.count_nodes();
	if(c>0) {
	    a=a+"\nPath to the current node in the XML document:\n";
	    a+=printNodePath(v)+"\n";
	}
	return a;
    }

    static final String printObjectPath(IDLDefCompiler.IDLObjectVector v) {
	String path="";
	int cnt=0;
	IDLObject obj;
	try {
	    while( (obj = v.getUpperObject(cnt)) != null ) {
		cnt++;
		String label = obj.getName("name")+" [ "+
		    obj.getName("id")+" ] { " + obj + " } \n";
		path = path + label;
	    }
	    return path;
	} catch(IDLflexException e) {}
	return "";
    }

    static final String printNodeName(Node n) {
	String s=n.getNodeName();
	try {
	    Element e=(Element)n;
	    NamedNodeMap nm= e.getAttributes();
	    int l=nm.getLength();
	    if(l>0) {
		s+="(";
		for(int i=0; i<l; i++) {
		    Node x = nm.item(i);
		    s+=(i>0?" ":"")+x.getNodeName()+"="+x.getNodeValue();
		}
		s+=")";
	    }
	} catch(Exception e) {}
	return s;
    }

    static final String printNodePath(IDLDefCompiler.IDLObjectVector v) {
	String text = "";
	try {
	    int nodes = v.count_nodes();
	    for(int i=0; i<nodes; i++) {
		Node n = v.nodeAt(i);
		text += printNodeName(n)+"\n";
	    }
	} catch(Exception e) {}

	Node n = v.nodeAt(v.count_nodes()-1);
        text += "\n-----XML path---\n"+printNodeName(n)+"\n";
        try {
            Node dad = n.getParentNode();
            while(dad!=null) {
                n=dad;
                text += printNodeName(n)+"\n";
                dad=n.getParentNode();
            }
        } catch(Exception e) {}
	return text;
    }
}
