/*
 * IDLflex -- A flexible, generic code generator for CORBA IDL
 *
 * Copyright (C) 2000  Hans Reiser, AspectIX Research Team
 *                     University of Erlangen-Nuernberg, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 * The full text of the license can be found in the LICENSE.txt file.
 *
 * $Id: PrimitiveObj.java,v 1.4 2001/09/07 09:20:44 reiser Exp $
 */

package org.aspectix.IDLflex.IRObj;

import org.omg.CORBA.*;
import org.aspectix.IDLflex.IDLflexException;

/** implementation object which represents the primitive IDL data
 *  types. 
 */
public class PrimitiveObj extends IDLObject {
    public int TCKindValue;
    public PrimitiveObj(org.omg.CORBA.IRObject iro) {
	super( iro ); 
	PrimitiveDef pd = PrimitiveDefHelper.narrow(iro);
	TCKindValue = pd.type().kind().value();
    }
    public void init_members() throws IDLflexException {
	super.init_members();
	switch(TCKindValue) {
	case TCKind._tk_boolean:
	    name = "tk_boolean"; break;
	case TCKind._tk_char:
	    name = "tk_char"; break;
	case TCKind._tk_wchar:
	    name = "tk_wchar"; break;
	case TCKind._tk_octet:
	    name = "tk_octet"; break;
	case TCKind._tk_string:
	    name = "tk_string"; break;
	case TCKind._tk_wstring:
	    name = "tk_wstring"; break;
	case TCKind._tk_short:
	    name = "tk_short"; break;
	case TCKind._tk_ushort:
	    name = "tk_ushort"; break;
	case TCKind._tk_long:
	    name = "tk_long"; break;
	case TCKind._tk_ulong:
	    name = "tk_ulong"; break;
	case TCKind._tk_longlong:
	    name = "tk_longlong"; break;
	case TCKind._tk_ulonglong:
	    name = "tk_ulonglong"; break;
	case TCKind._tk_float:
	    name = "tk_float"; break;
	case TCKind._tk_double:
	    name = "tk_double"; break;
	case TCKind._tk_fixed:
	    name = "tk_fixed"; break;
	case TCKind._tk_longdouble:
	    name = "tk_longdouble"; break;
	case TCKind._tk_TypeCode:
	    name = "tk_TypeCode"; break;
	case TCKind._tk_any:
            name = "tk_any"; break;
	case TCKind._tk_objref:
	    name = "tk_objref"; break;
	default:
	    name = "##UNSUPPORTED PRIMITIVE TCKIND VALUE "+TCKindValue+"##";
	}
    }

    public boolean getAttribute(String spec) throws IDLflexException {
	if(!spec.startsWith("tk")) return super.getAttribute(spec);
	return spec.equalsIgnoreCase(name);
    }

    public boolean is_a(String spec, boolean inherit) {
	if(spec.equalsIgnoreCase("PrimitiveObj")) return true;
	if(inherit) return super.is_a(spec, inherit);
	else return false;
    }
}
